@@ -104,7 +104,7 @@ fn test_loads_nan() {
 }
 
 #[test]
-/// Tests dumps method
+/// Tests dumps method consistency
 fn test_dumps() {
     let parsed_data = common::get_file_content_parsed(PARENT_FOLDER, "full.ura").unwrap();
     let string_data = dump(&parsed_data);
@@ -112,6 +112,49 @@ fn test_dumps() {
     assert_eq!(new_parsed_data, get_expected());
 }
 
+#[test]
+/// Tests dumps method result
+fn test_dumps_result() {
+    let str = "foo: [
+    bar:
+        baz: [
+            far: [
+                faz: \"foo\"
+            ],
+            far: \"faz\",
+            far: \"faz\"
+        ],
+    [empty, empty, empty],
+    [
+        foo:
+            hi: \"bar\"
+            jeje: [
+                foo: [
+                    bar:
+                        baz: [
+                            far: [
+                                faz: \"foo\"
+                            ],
+                            far: \"faz\",
+                            far: \"faz\"
+                        ],
+                    [empty, empty, empty],
+                    [
+                        foo:
+                            hi: \"bar\"
+                            jeje: []
+                    ]
+                ]
+            ]
+    ]
+]"
+    .to_string();
+
+    let parsed = parse(&str).unwrap();
+    let dumped = dump(&parsed);
+    assert_eq!(str, dumped);
+}
+
 #[test]
 /// Tests dumps method with NaNs values
 fn test_dumps_nan() {
